<div class="row overview-container">
    <div class="col-md-4">
        <div class="card text-white mb-3" style="background-color: #0d6efd;">
            <div class="card-header">Found / Scanned / Total</div>
            <div class="card-body">
                <h5 class="card-title">{{ data.devices_alive }} / {{ data.devices_scanned }} / {{ data.devices_total }}</h5>
            </div>
        </div>
    </div>
    <div class="col-md-4">
        <div class="card text-white mb-3" style="background-color: #198754;">
            <div class="card-header">Runtime</div>
            <div class="card-body">
                <h5 class="card-title">{{ data.run_time }}</h5>
            </div>
        </div>
    </div>
    <div class="col-md-4">
        <div class="card text-white mb-3" style="background-color: #dc3545;">
            <div class="card-header">Stage</div>
            <div class="card-body">
                <h5 class="card-title">{{ data.stage }}</h5>
            </div>
        </div>
    </div>
</div>
